﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.MemoryMappedFiles;
using System.Diagnostics;

namespace RealTimeChart_Music
{
	public class WaveReader
	{
		private WaveHeader header;
		private WaveFormat format;
		private WaveData data;
		private string filename;
		private MemoryMappedFile mmf;

		public WaveReader()
		{
			initFields();
			readWaveFromFile();
		}

		public WaveData Data
		{
			get { return data; }
		}
		public WaveFormat Format
		{
			get { return format; }
		}

		public WaveReader(string filename)
		{
			initFields();
			this.filename = filename;
			readWaveFromFile();
		}

		private void initFields()
		{
			header = new WaveHeader();
			format = new WaveFormat();
			data = new WaveData();
			filename = "";
		}

		public string Filename
		{
			set { this.filename = value; }
			get { return this.filename; }
		}

		public MemoryMappedFile File
		{
			get { return mmf; }
		}

		private void readWaveFromFile()
		{
			mmf = MemoryMappedFile.CreateFromFile(filename, FileMode.Open);
			using (var viewAccessor = mmf.CreateViewAccessor(0, 44))
			{
				byte[] bytes = new byte[4];
				viewAccessor.ReadArray<byte>(0, bytes,0,4);
				header.RiffType = System.Text.Encoding.Default.GetString(bytes);
				header.FileLength = viewAccessor.ReadUInt32(4);
				viewAccessor.ReadArray<byte>(8, bytes, 0, 4);
				header.GroupID = System.Text.Encoding.Default.GetString(bytes);
				viewAccessor.ReadArray<byte>(12, bytes, 0, 4);
				format.FormatID = System.Text.Encoding.Default.GetString(bytes);
				format.Size = viewAccessor.ReadUInt32(16);
				format.FormatTag = viewAccessor.ReadUInt16(20);
				format.Channels = viewAccessor.ReadUInt16(22);
				format.SamplesPerSecond = viewAccessor.ReadUInt32(24);
				format.AverageBytesPerSecond = viewAccessor.ReadUInt32(28);
				format.BlockAlign = viewAccessor.ReadUInt16(32);
				format.BitsPerSample = viewAccessor.ReadUInt16(34);
				viewAccessor.ReadArray<byte>(36, bytes, 0, 4);
				data.DataID = System.Text.Encoding.Default.GetString(bytes);
				data.Size = viewAccessor.ReadUInt32(40);

				Debug.Write(header.ToString() + format.ToString() + data.ToString());
			}
		}
		
		public uint Duration
		{
			get { return data.Size / format.AverageBytesPerSecond; }	
		}
	}

	internal class WaveHeader
	{
		private string sGroupID; // RIFF
		private uint dwFileLength; // total file length minus 8, which is taken up by RIFF
		private string sRiffType; // always WAVE

		/// <summary>
		/// Initializes a WaveHeader object with the default values.
		/// </summary>
		public WaveHeader()
		{
			dwFileLength = 0;
			sGroupID = "RIFF";
			sRiffType = "WAVE";
		}

		public string GroupID
		{
			get { return sGroupID; }
			set { sGroupID = value; }
		}

		public uint FileLength
		{
			get { return dwFileLength; }
			set { dwFileLength = value; }
		}

		public string RiffType
		{
			get { return sRiffType; }
			set { sRiffType = value; }
		}

		public override string ToString()
		{
			return string.Format("WAVE HEADER\r\nGroup ID: {0}\r\nFile Length: {1}\r\nRiff Type: {2}\r\n\r\n",
				sGroupID, dwFileLength, sRiffType);
		}
	}

	public class WaveFormat
	{
		private string sChunkID;         // Four bytes: "fmt "
		private uint dwChunkSize;        // Length of header in bytes
		private ushort wFormatTag;       // 1 (MS PCM)
		private ushort wChannels;        // Number of channels
		private uint dwSamplesPerSec;    // Frequency of the audio in Hz... 44100
		private uint dwAvgBytesPerSec;   // for estimating RAM allocation
		private ushort wBlockAlign;      // sample frame size, in bytes
		private ushort wBitsPerSample;    // bits per sample

		public string FormatID
		{
			get { return sChunkID; }
			set { sChunkID = value; }
		}
		public uint Size
		{
			get { return dwChunkSize; }
			set { dwChunkSize = value; }
		}
		public ushort FormatTag
		{
			get { return wFormatTag; }
			set { wFormatTag = value; }
		}
		public ushort Channels
		{
			get { return wChannels; }
			set { wChannels = value; }
		}
		public uint SamplesPerSecond
		{
			get { return dwSamplesPerSec; }
			set { dwSamplesPerSec = value; }
		}
		public uint AverageBytesPerSecond
		{
			get { return dwAvgBytesPerSec; }
			set { dwAvgBytesPerSec = value; }
		}
		public ushort BlockAlign
		{
			get { return wBlockAlign; }
			set { wBlockAlign = value; }
		}
		public ushort BitsPerSample
		{
			get { return wBitsPerSample; }
			set { wBitsPerSample = value; }
		}
		/// <summary>
		/// Initializes a format chunk with the following properties:
		/// Sample rate: 44100 Hz
		/// Channels: Stereo
		/// Bit depth: 16-bit
		/// </summary>
		public WaveFormat()
		{
			sChunkID = "fmt ";
			dwChunkSize = 16;
			wFormatTag = 1;
			wChannels = 2;
			dwSamplesPerSec = 44100;
			wBitsPerSample = 16;
			wBlockAlign = (ushort)(wChannels * (wBitsPerSample / 8));
			dwAvgBytesPerSec = dwSamplesPerSec * wBlockAlign;
		}
		public override string ToString()
		{
			return string.Format("WAVE FORMAT\r\nID: {0}\r\nSize: {1}\r\nFormat Tag: {2}\r\nChannels: {3}\r\nSamples Per Second: {4}\r\nBits Per Sample: {5}\r\nBlock Align: {6}\r\nAverage Bytes Per Second: {7}\r\n\r\n",
				sChunkID, dwChunkSize, wFormatTag, wChannels, dwSamplesPerSec, wBitsPerSample, wBlockAlign, dwAvgBytesPerSec);
		}
	}

	public class WaveData
	{
		private string sChunkID;     // "data"
		private uint dwChunkSize;    // Length of header in bytes

		public string DataID
		{
			get { return sChunkID; }
			set { sChunkID = value; }
		}
		public uint Size
		{
			get { return dwChunkSize; }
			set { dwChunkSize = value; }
		}

		/// <summary>
		/// Initializes a new data chunk with default values.
		/// </summary>
		public WaveData()
		{
			dwChunkSize = 0;
			sChunkID = "data";
		}


		public string ToString()
		{
			return string.Format("WAVE DATA\r\nID: {0}\r\nSize: {1}\r\n", sChunkID, dwChunkSize);
		}
	}
}
